@@ -13,7 +13,7 @@
 #' @examples
 #' ## load existing workbook from package folder
 #' wb <- loadWorkbook(xlsxFile = system.file("loadExample.xlsx", package= "openxlsx"))
-#' sheets(wb)  #list worksheets
+#' names(wb)  #list worksheets
 #' wb ## view object
 #' ## Add a worksheet
 #' addWorksheet(wb, "A new worksheet")
@@ -26,14 +26,13 @@
 #' ## Save workbook
 #' saveWorkbook(wb, "loadExample.xlsx", overwrite = TRUE)
 loadWorkbook <- function(xlsxFile){
-
+  
   
   if(!file.exists(xlsxFile))
     stop("File does not exist.")
   
   wb <- createWorkbook()
   
-  
   ## create temp dir
   xmlDir <- paste0(tempdir(),  tempfile(tmpdir = ""), "_openxlsx_loadWorkbook")
   
@@ -70,8 +69,8 @@ loadWorkbook <- function(xlsxFile){
     workbook <-  removeHeadTag(workbook)
     
     sheets <- unlist(regmatches(workbook, gregexpr("<sheet .*/sheets>", workbook, perl = TRUE)))
-    sheetrId <- as.numeric(unlist(regmatches(sheets, gregexpr('(?<=r:id="rId)[0-9]+', sheets, perl = TRUE))))
-    sheetId <- as.numeric(unlist(regmatches(sheets, gregexpr('(?<=sheetId=")[0-9]+', sheets, perl = TRUE))))
+    sheetrId <- as.integer(unlist(regmatches(sheets, gregexpr('(?<=r:id="rId)[0-9]+', sheets, perl = TRUE))))
+    sheetId <- as.integer(unlist(regmatches(sheets, gregexpr('(?<=sheetId=")[0-9]+', sheets, perl = TRUE))))
     sheetNames <- unlist(regmatches(sheets, gregexpr('(?<=name=")[^"]+', sheets, perl = TRUE)))
     
     calcPr <- .Call("openxlsx_getChildlessNode", workbook, "<calcPr ", PACKAGE = "openxlsx")
@@ -82,10 +81,9 @@ loadWorkbook <- function(xlsxFile){
     sheetNames <- sheetNames[order(sheetrId)]
     sheetNames <- replaceXMLEntities(sheetNames)
     
-    
     ## add worksheets to wb
     invisible(lapply(sheetNames, function(sheetName) wb$addWorksheet(sheetName)))
-      
+    
     ## defined Names
     dNames <- .Call("openxlsx_getNodes", workbook, "<definedNames>", PACKAGE = "openxlsx")
     if(length(dNames) > 0)
@@ -98,8 +96,8 @@ loadWorkbook <- function(xlsxFile){
     
     sharedStrings <- readLines(sharedStringsXML, warn = FALSE, encoding="UTF-8")
     sharedStrings <- removeHeadTag(sharedStrings)
-      
-    uniqueCount <- as.numeric(regmatches(sharedStrings, regexpr('(?<=uniqueCount=")[0-9]+', sharedStrings, perl = TRUE)))
+    
+    uniqueCount <- as.integer(regmatches(sharedStrings, regexpr('(?<=uniqueCount=")[0-9]+', sharedStrings, perl = TRUE)))
     
     ## read in and get <si> nodes
     vals <- .Call("openxlsx_getNodes", sharedStrings, "<si>", PACKAGE = "openxlsx")
@@ -108,7 +106,7 @@ loadWorkbook <- function(xlsxFile){
     wb$sharedStrings <- vals
     
   }
-    
+  
   ## xl\styles
   if(length(stylesXML) > 0){
     
@@ -145,7 +143,7 @@ loadWorkbook <- function(xlsxFile){
     
     fills <- .Call("openxlsx_getNodes", styles, "<fill>", PACKAGE = "openxlsx")
     fills <- lapply(fills, nodeAttributes)
-
+    
     borders <- .Call("openxlsx_getNodes", styles, "<border>", PACKAGE = "openxlsx")     
     borders <- sapply(borders, buildBorder, USE.NAMES = FALSE)
     
@@ -162,13 +160,13 @@ loadWorkbook <- function(xlsxFile){
     styleObjects <- list()
     flag <- FALSE
     for(s in xfVals){
-
+      
       style <- createStyle()
       if(any(s != "0")){
-                
+        
         if(s[["fontId"]] != "0"){
-          thisFont <- fonts[[(as.numeric(s[["fontId"]])+1)]]
-
+          thisFont <- fonts[[(as.integer(s[["fontId"]])+1)]]
+          
           if("sz" %in% names(thisFont))
             style$fontSize <- thisFont$sz
           
@@ -200,7 +198,7 @@ loadWorkbook <- function(xlsxFile){
         
         
         if(s[["numFmtId"]] != "0" & numFmtFlag){
-          if(as.numeric(s[["numFmtId"]]) < 164){
+          if(as.integer(s[["numFmtId"]]) < 164){
             style$numFmt <- list(numFmtId = s[["numFmtId"]])
           }else{
             style$numFmt <- numFmts[[which(s[["numFmtId"]] == numFmtsIds)]]
@@ -208,9 +206,9 @@ loadWorkbook <- function(xlsxFile){
         }
         
         ## Border
-        if(s[["borderId"]] != "0" & "applyBorder" %in% names(s)){
-     
-          thisBorder <- borders[[as.numeric(s[["borderId"]]) + 1]]
+        if(s[["borderId"]] != "0"){# & "applyBorder" %in% names(s)){
+          
+          thisBorder <- borders[[as.integer(s[["borderId"]]) + 1L]]
           
           if("borderLeft" %in% names(thisBorder)){
             style$borderLeft    <- thisBorder$borderLeft
@@ -231,28 +229,28 @@ loadWorkbook <- function(xlsxFile){
             style$borderBottom    <- thisBorder$borderBottom
             style$borderBottomColour <- thisBorder$borderBottomColour
           }
-            
+          
         }
         
         ## alignment
-        if("applyAlignment" %in% names(s) & "horizontal" %in% names(s))
+        applyAlignment <- "applyAlignment" %in% names(s)
+        if("horizontal" %in% names(s))# & applyAlignment)
           style$halign <- s[["horizontal"]]
         
-        if("applyAlignment" %in% names(s) & "vertical" %in% names(s))
+        if("vertical" %in% names(s))
           style$valign <- s[["vertical"]]  
         
-        if("applyAlignment" %in% names(s) & "textRotation" %in% names(s))
+        if("textRotation" %in% names(s))
           style$textRotation <- s[["textRotation"]]
-          
+        
         ## wrap text
         if("wrapText" %in% names(s)){
           if(s[["wrapText"]] %in% c("1", "true"))
             style$wrapText <- TRUE
         }
         
-        if(s[["fillId"]] != "0" && "applyFill" %in% names(s)){
-          
-          fillId <- as.numeric(s[["fillId"]]) + 1
+        if(s[["fillId"]] != "0"){# && "applyFill" %in% names(s)){
+          fillId <- as.integer(s[["fillId"]]) + 1L
           
           tmpFg <- fills[[fillId]]$fgColor
           tmpBg <- fills[[fillId]]$bgColor
@@ -323,7 +321,7 @@ loadWorkbook <- function(xlsxFile){
     
     wb$queryTables <- lapply(sort(queryTablesXML), function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx")))
     wb$Content_Types <- c(wb$Content_Types, 
-                          sprintf('<Override PartName="/xl/queryTables/queryTable1.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.queryTable+xml"/>', 1:length(queryTablesXML)))   
+                          sprintf('<Override PartName="/xl/queryTables/queryTable%s.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.queryTable+xml"/>', 1:length(queryTablesXML)))   
   }
   
   
@@ -333,36 +331,36 @@ loadWorkbook <- function(xlsxFile){
     wb$workbook.xml.rels <- c(wb$workbook.xml.rels, '<Relationship Id="rId3" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/connections" Target="connections.xml"/>')
     wb$Content_Types <- c(wb$Content_Types, '<Override PartName="/xl/connections.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.connections+xml"/>')
   }
-
+  
   ## externalLinks
-  if(length(extLinksXML)){
+  if(length(extLinksXML) > 0){
     wb$externalLinks <- lapply(sort(extLinksXML), function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx")))
     
     wb$Content_Types <-c(wb$Content_Types, 
-        sprintf('<Override PartName="/xl/externalLinks/externalLink%s.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.externalLink+xml"/>', 1:length(extLinksXML)))
-
+                         sprintf('<Override PartName="/xl/externalLinks/externalLink%s.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.externalLink+xml"/>', 1:length(extLinksXML)))
+    
     wb$workbook.xml.rels <- c(wb$workbook.xml.rels, sprintf('<Relationship Id="rId4" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/externalLink" Target="externalLinks/externalLink1.xml"/>',
                                                             1:length(extLinksXML)))
   }
-    
-    
+  
   ## externalLinksRels
-  if(length(extLinksRelsXML))
+  if(length(extLinksRelsXML) > 0)
     wb$externalLinksRels <- lapply(sort(extLinksRelsXML), function(x) removeHeadTag(.Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx")))
-    
+  
   
   ##*----------------------------------------------------------------------------------------------*##
   ### BEGIN READING IN WORKSHEET DATA
   ##*----------------------------------------------------------------------------------------------*##
   
   ## xl\worksheets
-  wsNumber <- as.numeric(regmatches(worksheetsXML, regexpr("(?<=sheet)[0-9]+(?=\\.xml)", worksheetsXML, perl = TRUE)))
+  wsNumber <- as.integer(regmatches(worksheetsXML, regexpr("(?<=sheet)[0-9]+(?=\\.xml)", worksheetsXML, perl = TRUE)))
   worksheetsXML <- worksheetsXML[order(wsNumber)]
   
   ws <- lapply(worksheetsXML, function(x) readLines(x, warn = FALSE, encoding = "UTF-8"))
   ws <- lapply(ws, removeHeadTag)
   wsTemp <- lapply(ws, function(x) strsplit(x, split = "<sheetData>")[[1]])
-    
+  
+  ## If there need to split at sheetData tag
   noData <- grepl("<sheetData/>", ws)
   if(any(noData))
     wsTemp[noData] <- lapply(ws[noData], function(x) strsplit(x, split = "<sheetData/>")[[1]])                            
@@ -383,11 +381,11 @@ loadWorkbook <- function(xlsxFile){
   cells[noData] <- "No Data"
   
   rowNumbers <- styleRefs <- s <- replicate(NULL, n = nSheets)
-
+  
   for(i in 1:length(cells)){
     
     if(!noData[[i]]){
-    
+      
       rowNumbers[[i]] <- .Call("openxlsx_getRefs", rows[[i]], 1, PACKAGE = "openxlsx")
       
       r <- .Call("openxlsx_getRefs", cells[[i]], 1, PACKAGE = "openxlsx")
@@ -412,8 +410,9 @@ loadWorkbook <- function(xlsxFile){
       cells[[i]] <- cells[[i]][which(grepl(' s="', cells[[i]]))]
       s[[i]] <- .Call("openxlsx_getCellStyles", cells[[i]], PACKAGE = "openxlsx")
       styleRefs[[i]] <- .Call("openxlsx_getRefs", cells[[i]], 1, PACKAGE = "openxlsx")
-    
+      
     }
+    
     ## row heights
     if(length(rows[[i]]) > 0){
       customRowHeights <- .Call("openxlsx_getAttr", rows[[i]], ' ht="', PACKAGE = "openxlsx")
@@ -426,8 +425,8 @@ loadWorkbook <- function(xlsxFile){
     cols <- cols[grepl("customWidth", cols)]
     if(length(cols) > 0){
       
-      mins <- as.numeric(.Call("openxlsx_getAttr", cols, 'min="', PACKAGE = "openxlsx"))
-      max <- as.numeric(.Call("openxlsx_getAttr", cols, 'max="', PACKAGE = "openxlsx"))
+      mins <- as.integer(.Call("openxlsx_getAttr", cols, 'min="', PACKAGE = "openxlsx"))
+      max <- as.integer(.Call("openxlsx_getAttr", cols, 'max="', PACKAGE = "openxlsx"))
       widths <-  .Call("openxlsx_getAttr", cols, 'width="', PACKAGE = "openxlsx") 
       
       if(any(mins != max)){
@@ -489,27 +488,27 @@ loadWorkbook <- function(xlsxFile){
     
     if(length(drawingId) > 0)
       wb$worksheets[[i]]$drawing <- gsub('rId.+"', 'rId1"', drawingId)
-
+    
   }
-
+  
   
   ## styles
   for(i in 1:nSheets){
-  
+    
     ## Sheet Data
     if(length(s[[i]]) > 0){
-    
+      
       ## write style coords to styleObjects (will be in order)
       sx <- s[[i]]
       
       if(any(!is.na(sx))){
-        sRef <- lapply(as.numeric(unique(sx[!is.na(sx)])), function(styleInd){ 
+        sRef <- lapply(as.integer(unique(sx[!is.na(sx)])), function(styleInd){ 
           
           if(styleInd > 0){
             sRef <- styleRefs[[i]][sx == styleInd & !is.na(sx)]        
-            rows <- as.numeric(gsub("[^0-9]", "", sRef))
+            rows <- as.integer(gsub("[^0-9]", "", sRef))
             cols <- .Call("openxlsx_RcppConvertFromExcelRef", sRef)
-                                                
+            
             wb$styleObjects[[styleInd]]$cells <- append(wb$styleObjects[[styleInd]]$cells, 
                                                         list(list(sheet = wb$getSheetName(1:nSheets)[[i]],
                                                                   rows = rows,
@@ -529,8 +528,8 @@ loadWorkbook <- function(xlsxFile){
   if(length(sheetRelsXML) > 0){
     
     sheetRelsXML <- sheetRelsXML[order(nchar(sheetRelsXML), sheetRelsXML)]
-    sheetNumber <- as.numeric(regmatches(sheetRelsXML, regexpr("(?<=sheet)[0-9]+(?=\\.xml)", sheetRelsXML, perl = TRUE)))
-
+    sheetNumber <- as.integer(regmatches(sheetRelsXML, regexpr("(?<=sheet)[0-9]+(?=\\.xml)", sheetRelsXML, perl = TRUE)))
+    
     xml <- lapply(sheetRelsXML, readLines, warn = FALSE)
     xml <- unlist(lapply(xml, removeHeadTag))
     xml <- gsub("<Relationships .*?>", "", xml)
@@ -539,11 +538,11 @@ loadWorkbook <- function(xlsxFile){
     
     ## tables
     if(length(tablesXML) > 0){
-  
-      tables <- lapply(xml, function(x) as.numeric(regmatches(x, regexpr("(?<=table)[0-9]+(?=\\.xml)", x, perl = TRUE))))
+      
+      tables <- lapply(xml, function(x) as.integer(regmatches(x, regexpr("(?<=table)[0-9]+(?=\\.xml)", x, perl = TRUE))))
       if(length(unlist(tables)) > 0){  
         ## get the tables that belong to each worksheet and create a worksheets_rels for each
-        tCount <- 2 ## table r:Ids start at 2
+        tCount <- 2L ## table r:Ids start at 2
         for(i in 1:length(tables)){
           k <- 1:length(tables[[i]]) + tCount
           wb$worksheets_rels[[sheetNumber[i]]] <- unlist(c(wb$worksheets_rels[[sheetNumber[i]]],
@@ -569,7 +568,7 @@ loadWorkbook <- function(xlsxFile){
     hlinksInds <- which(sapply(hlinks, length) > 0)
     
     if(length(hlinksInds) > 0){
-    
+      
       hlinks <- hlinks[hlinksInds]
       for(i in 1:length(hlinksInds)){
         targets <- unlist(lapply(hlinks[[i]], function(x) regmatches(x, gregexpr('(?<=Target=").*?"', x, perl = TRUE))[[1]]))
@@ -585,7 +584,7 @@ loadWorkbook <- function(xlsxFile){
     drawInds <- which(sapply(draw, length) > 0)
     
     if(length(drawingRelsXML) > 0){
-
+      
       dRels <- lapply(drawingRelsXML, readLines, warn = FALSE)
       dRels <- unlist(lapply(dRels, removeHeadTag))
       dRels <- gsub("<Relationships .*?>", "", dRels)
@@ -601,7 +600,7 @@ loadWorkbook <- function(xlsxFile){
       ## split at one/two cell Anchor
       dXML <- regmatches(dXML, gregexpr("<xdr:...CellAnchor.*?</xdr:...CellAnchor>", dXML))
     } 
-
+    
     if(length(drawInds) > 0){
       
       draw <- draw[drawInds]
@@ -621,19 +620,19 @@ loadWorkbook <- function(xlsxFile){
         drawingInd <- grepl(target, drawingsXML)
         if(any(drawingInd))
           wb$drawings[dSheet] <- dXML[drawingInd]
-      
+        
       }     
     }
   } 
-
+  
   ## table rels
   if(length(tableRelsXML) > 0){
     
     tableRelsXML <- sort(tableRelsXML)
     wb$tables.xml.rels <- character(length=length(tablesXML))
-
+    
     ## which sheet does it belong to
-    inds <- as.numeric(regmatches(tableRelsXML, regexpr("(?<=table)[0-9]+(?=\\.xml.rels$)", tableRelsXML, perl = TRUE)))
+    inds <- as.integer(regmatches(tableRelsXML, regexpr("(?<=table)[0-9]+(?=\\.xml.rels$)", tableRelsXML, perl = TRUE)))
     xml <- sapply(tableRelsXML, function(x) .Call("openxlsx_cppReadFile", x, PACKAGE = "openxlsx"), USE.NAMES = FALSE)
     xml <- sapply(xml, removeHeadTag, USE.NAMES = FALSE)
     wb$tables.xml.rels[inds] <- xml
@@ -641,200 +640,8 @@ loadWorkbook <- function(xlsxFile){
   }else if(length(tablesXML) > 0){
     wb$tables.xml.rels <- rep("", length(tablesXML))
   }
-
+  
   return(wb)
   
 }
 
-
-getAttrs <- function(xml, tag){
-  
-  x <- lapply(xml, function(x) .Call("openxlsx_getChildlessNode", x, tag, PACKAGE = "openxlsx"))
-  x[sapply(x, length) == 0] <- ""
-  a <- lapply(x, function(x) regmatches(x, regexpr('[a-zA-Z]+=".*?"', x)))
-  
-  names = lapply(a, function(xml) regmatches(xml, regexpr('[a-zA-Z]+(?=\\=".*?")', xml, perl = TRUE)))
-  vals =  lapply(a, function(xml) regmatches(xml, regexpr('(?<=").*?(?=")', xml, perl = TRUE)))
-  names(vals) <- names
-  return(vals)
-  
-}
-
-
-buildFontList <- function(fonts){
-  
-  sz <- getAttrs(fonts, "<sz ")
-  colour <- getAttrs(fonts, "<color ")
-  name <- getAttrs(fonts, "<name ")
-  family <- getAttrs(fonts, "<family ")
-  scheme <- getAttrs(fonts, "<scheme ")
-  
-  italic <- lapply(fonts, function(x) .Call("openxlsx_getChildlessNode", x, "<i", PACKAGE = "openxlsx"))
-  bold <- lapply(fonts, function(x) .Call("openxlsx_getChildlessNode", x, "<b", PACKAGE = "openxlsx"))
-  underline <- lapply(fonts, function(x) .Call("openxlsx_getChildlessNode", x, "<u", PACKAGE = "openxlsx"))
-  
-  ## Build font objects
-  ft <- replicate(list(), n=length(fonts))
-  for(i in 1:length(fonts)){
-    
-    f <- NULL
-    nms <- NULL
-    if(length(unlist(sz[i])) > 0){
-      f <- c(f, sz[i])
-      nms <- c(nms, "sz")     
-    }
-    
-    if(length(unlist(colour[i])) > 0){
-      f <- c(f, colour[i])
-      nms <- c(nms, "color")  
-    }
-    
-    if(length(unlist(name[i])) > 0){
-      f <- c(f, name[i])
-      nms <- c(nms, "name") 
-    }
-    
-    if(length(unlist(family[i])) > 0){
-      f <- c(f, family[i])
-      nms <- c(nms, "family") 
-    }
-    
-    if(length(unlist(scheme[i])) > 0){
-      f <- c(f, scheme[i])
-      nms <- c(nms, "scheme") 
-    }
-    
-    if(length(italic[[i]]) > 0){
-      f <- c(f, "italic")
-      nms <- c(nms, "italic") 
-    }
-    
-    if(length(bold[[i]]) > 0){
-      f <- c(f, "bold")
-      nms <- c(nms, "bold") 
-    }
-    
-    if(length(underline[[i]]) > 0){
-      f <- c(f, "underline")
-      nms <- c(nms, "underline") 
-    }
-    
-    f <- lapply(1:length(f), function(i) unlist(f[i]))
-    names(f) <- nms
-    
-    ft[[i]] <- f
-    
-  }
-  
-  ft
-  
-}
-
-
-nodeAttributes <- function(x){
-  
-  
-  x <- paste0("<", unlist(strsplit(x, split = "<")))
-  x <- x[grepl("<bgColor|<fgColor", x)]
-  
-  if(length(x) == 0)
-    return("")
-  
-  attrs <- regmatches(x, gregexpr(' [a-zA-Z]+="[^"]*', x, perl =  TRUE))
-  tags <- regmatches(x, gregexpr('<[a-zA-Z]+ ', x, perl =  TRUE))
-  tags <- lapply(tags, gsub, pattern = "<| ", replacement = "")  
-  attrs <- lapply(attrs, gsub, pattern = '"', replacement = "")      
-  
-  attrs <- lapply(attrs, strsplit, split = "=")
-  for(i in 1:length(attrs)){
-    nms <- lapply(attrs[[i]], "[[", 1)
-    vals <- lapply(attrs[[i]], "[[", 2)
-    a <- unlist(vals)
-    names(a) <- unlist(nms)
-    attrs[[i]] <- a
-  }
-  
-  names(attrs) <- unlist(tags)
-  
-  attrs
-}
-
-
-buildBorder <- function(x){
-  
-  ## gets all borders that have children
-  x <- unlist(lapply(c("<left", "<right", "<top", "<bottom"), function(tag) .Call("openxlsx_getNodes", x, tag, PACKAGE = "openxlsx")))
-  if(length(x) == 0)
-    return(NULL)
-    
-  sides <- c("TOP", "BOTTOM", "LEFT", "RIGHT")
-  sideBorder <- character(length=length(x))
-  for(i in 1:length(x)){
-    tmp <- sides[sapply(sides, function(s) grepl(s, x[[i]], ignore.case = TRUE))]
-    if(length(tmp) > 1) tmp <- tmp[[1]]
-    if(length(tmp) == 1)
-      sideBorder[[i]] <- tmp
-  }
-  
-  sideBorder <- sideBorder[sideBorder != ""]
-  x <- x[sideBorder != ""]
-  if(length(sideBorder) == 0)
-    return(NULL)
-  
-  
-  ## style
-  weight <- gsub('style=|"', "", regmatches(x, regexpr('style="[a-z]+"', x, perl = TRUE)))
-    
-  ## Colours
-  cols <- replicate(n = length(sideBorder), list(rgb = "FF000000"))
-  colNodes <- unlist(sapply(x, function(xml) .Call("openxlsx_getChildlessNode", xml, "<color", PACKAGE = "openxlsx"), USE.NAMES = FALSE))
-
-  if(length(colNodes) > 0){
-    attrs <- regmatches(colNodes, regexpr('(theme|indexed|rgb)=".+"', colNodes))
-  }else{
-    attrs <- NULL
-  }
-    
-  if(length(attrs) != length(x)){
-   return(
-     list("borders" = paste(sideBorder, collapse = ""),
-          "colour" = cols)
-     ) 
-  }
-  
-  attrs <- strsplit(attrs, split = "=")
-  cols <- sapply(attrs, function(attr){
-    y <- list(gsub('"', "", attr[[2]]))
-    names(y) <- gsub(" ", "", attr[[1]])
-    y
-  })
-  
-  ## sideBorder & cols
-  style <- list()
-  
-  if("LEFT" %in% sideBorder){
-    style$borderLeft <- weight[which(sideBorder == "LEFT")]
-    style$borderLeftColour <- cols[which(sideBorder == "LEFT")]
-  }
-  
-  if("RIGHT" %in% sideBorder){
-    style$borderRight <- weight[which(sideBorder == "RIGHT")]
-    style$borderRightColour <- cols[which(sideBorder == "RIGHT")]
-  }
-  
-  if("TOP" %in% sideBorder){
-    style$borderTop <- weight[which(sideBorder == "TOP")]
-    style$borderTopColour <- cols[which(sideBorder == "TOP")]
-  }
-  
-  if("BOTTOM" %in% sideBorder){
-    style$borderBottom <- weight[which(sideBorder == "BOTTOM")]
-    style$borderBottomColour <- cols[which(sideBorder == "BOTTOM")]
-  }
-  
-  return(style)
-}
-
-
-
-
